/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Paste and Go.
 *
 * The Initial Developer of the Original Code is Jens Bannmann.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Jens Bannmann <jens.b@web.de>
 *  Oliver Aeberhard <aeberhard@gmx.ch>
 *  Holio Lin <gaghiel@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var pgInitDone = false;

// clipboard object
const pgClipboard = Components.classes["@mozilla.org/widget/clipboard;1"]
    .createInstance(Components.interfaces.nsIClipboard);
const pgTrans = Components.classes["@mozilla.org/widget/transferable;1"]
    .createInstance(Components.interfaces.nsITransferable);
pgTrans.addDataFlavor("text/unicode");

// prefix extensions.pasteandgo.
const pgPref = Components.classes["@mozilla.org/preferences-service;1"]
    .getService(Components.interfaces.nsIPrefService)
    .getBranch("extensions.pasteandgo2.");

var pgUrlbar;
var pgSearchbar;
var pgPlaceAtBottom = false;
var pgOpenNewTab = false;
var pgBackgroundTab = false;
var pgPreferSelectionClipboard = true;

// Determines if the PasteAndGo hotkeys have a higher priority than other hotkeys.
var pgHighPriority = false;

var installedGoKey;
var installedGoCommand;
var installedGoLabel;

var installedSearchKey;
var installedSearchCommand;
var installedSearchLabel;

window.addEventListener('load', function(e) { pgStartup(); }, false);

function pgStartup() {
    if (pgInitDone) return;
    pgInitDone = true;

    var pgGoKey;
    var pgSearchKey;
    var pgGoAccelKeys;
    var pgSearchAccelKeys;
    
    // load pref
    pgPlaceAtBottom = pgPref.getBoolPref("PlaceAtBottom");
    pgGoKey = pgPref.getCharPref("GoHotkey");
    pgSearchKey = pgPref.getCharPref("SearchHotkey");
    pgGoAccelKeys = pgPref.getCharPref("GoAccelKeys");
    pgSearchAccelKeys = pgPref.getCharPref("SearchAccelKeys");
    pgOpenNewTab = pgPref.getBoolPref("OpenNewTab");
    pgBackgroundTab = pgPref.getBoolPref("BackgroundTab");
    pgHighPriority = pgPref.getBoolPref("HighPriority");
    pgPreferSelectionClipboard = pgPref.getBoolPref("PreferSelectionClipboard");
    
    // bind urlbar
    pgUrlbar = document.getElementById("urlbar");
    if (pgUrlbar) {
        installedGoKey = dockElement("keyset", "key", "PasteAndGoKey",
                new Array("modifiers",   "key",   "command"  ),
                new Array(pgGoAccelKeys, pgGoKey, "PasteAndGoCommand"));
        installedGoCommand = dockElement("commandset", "command", "PasteAndGoCommand",
                new Array("oncommand"),
                new Array("PasteAndGo(false);"));
        pgUrlbar.addEventListener("popupshowing", pgInstallGo, true); // TODO: check is if need install on evry time pop up
    }

    // bind searchbar
    // Get the searchbar textbox
    // In firefox versions before 1.0 the id was "search-bar".
    if (document.getElementById("search-bar")) {
        pgSearchbar = document.getElementById("search-bar");
    }
    // As of 1.0 the id is "searchbox"
    else if (document.getElementById("searchbar")) {
        pgSearchbar = document.getElementById("searchbar");
    }

    if (pgSearchbar) {
        installedSearchKey = dockElement("keyset", "key", "PasteAndSearchKey",
                new Array("modifiers",   "key",       "command"  ),
                new Array(pgSearchAccelKeys, pgSearchKey, "PasteAndSearchCommand"));
        installedSearchCommand = dockElement("commandset", "command", "PasteAndSearchCommand",
                new Array("oncommand"),
                new Array("PasteAndGo(true);"));
        pgSearchbar.addEventListener("popupshowing", pgInstallSearch, true); // TODO: check is if need install on evry time pop up
    }
}

function pgInstallGo(e) {
    installedGoLabel = pgInstallMenuItem(e, "PasteAndGoMenuitem", "Paste and Go",
            "PasteAndGoCommand");
}

function pgInstallSearch(e) {
    installedSearchLabel = pgInstallMenuItem(e, "PasteAndSearchMenuitem", "Paste and Search",
            "PasteAndSearchCommand");
}

function pgInstallMenuItem(e, aID, aLabel, aCommand) {
    if (e.originalTarget.localName == "menupopup" // 1. event trigged by menupopup
            && !document.getElementById(aID)) {       // 2. avoid re-install
        var mi = document.createElement("menuitem");
        mi.setAttribute("id", aID);
        mi.setAttribute("label", aLabel);
        mi.setAttribute("command", aCommand);
        if (pgPlaceAtBottom) { // place at bottom
            e.originalTarget.appendChild(document.createElement("menuseparator"));
            e.originalTarget.appendChild(mi);
        } else {               // else place next to paste
            var items = e.originalTarget.childNodes;
            for (var i=0; i < items.length; i++) {
                var cmd = items[i].getAttribute("cmd");
                if(cmd == "cmd_paste") { // find out paste
                    e.originalTarget.insertBefore(mi, items[i+1]);
                    break;
                }
            }
        }
        
        return mi;
    }
    
    return null;
}

// for assign high priority hot key
// create type whos id, pref as parameter said, and dock it under first setType in document
function dockElement(setType, type, id, aNames, aValues) {
    var sets = document.getElementsByTagName(setType);
    var set;
    if (sets && sets.length > 0) {
        set = sets[0];
    } else {
        set = document.createElement(setType);
        document.documentElement.appendChild(set);
    }
    /*create a t with type 'type', let t.id = 'id'
      for each aNames, say a,
      each aValues, say v:
      t.a = v*/
    var element = document.createElement(type);
    element.setAttribute("id", id);
    for (i=0; i < aNames.length; i++) {
        element.setAttribute(aNames[i], aValues[i]);
    }

    // Set position in the keyset table. (If two commands share the
    // same hotkey configuration, only the topmost in the keyset table
    // will be executed)
    if (pgHighPriority) {
        set.insertBefore(element, set.firstChild);
    } else {
        set.appendChild(element);
    }
    
    return element;
}

function pgGetClipboard(which) {
    pgClipboard.getData(pgTrans, which);

    var str = new Object();
    var strLength = new Object();

    try {
        pgTrans.getTransferData("text/unicode", str, strLength);
    } catch (e) {
        str = null;
    }

    if (str) {
        if ("nsISupportsWString" in Components.interfaces) {
            str = str.value.QueryInterface(Components.interfaces.nsISupportsWString);
        } else if ("nsISupportsString" in Components.interfaces) {
            str = str.value.QueryInterface(Components.interfaces.nsISupportsString);
        } else {
            str = null;
        }
    }

    return str;
}

function PasteAndGo(doSearch) {
    var globalClipboard = pgGetClipboard(pgClipboard.kGlobalClipboard);
    var str = globalClipboard;

    // On Linux, use either selection or global clipboard according to pref and situation
    if (pgClipboard.supportsSelectionClipboard()) {
        var selectionClipboard = pgGetClipboard(pgClipboard.kSelectionClipboard);

        if (selectionClipboard && (pgPreferSelectionClipboard || !globalClipboard)) {
            str = selectionClipboard;
        }
    }

    //pgOpenNewTab = pgPref.getBoolPref("OpenNewTab");
    var shouldOpenNewTab = (pgOpenNewTab && window._content.document.location.href != "about:blank");

    if (doSearch) {
        // put the query in the search bar
        pgSearchbar.value = str;
        if (pgSearchbar.searchValue) {
            // in old builds (up to 0.7), searchValue's getter is so kind and gives
            // us the URL to load:
            str = pgSearchbar.searchValue;
            pgUrlbar.setAttribute("value", str);
            loadURI(str);

            if (shouldOpenNewTab) {
                var browser = getBrowser();
                var tab = browser.addTab(str);
                browser.selectedTab = tab;
            } else {
                loadURI(str);
            }

        // after 0.7
        } else {
            pgSearchbar.doSearch(str, shouldOpenNewTab);
        }

    } else {
        pgUrlbar.setAttribute("value", str);
        if (shouldOpenNewTab) {
            var browser = getBrowser();
            var tab = browser.addTab(str);
            if (!pgBackgroundTab) {
                browser.selectedTab = tab;
            }
        } else {
            loadURI(str);
        }
    }
}
